from abc import ABC, abstractmethod
import json
import logging

logger = logging.getLogger(__name__)

class AITools(ABC):

    def call_function(self, function_name, call_id, args):
        # get method by name if it exists
        if hasattr(self, function_name):
            meth = getattr(self, function_name)
            # check that method exists
            if meth:
                if args is None:
                    # call method without args
                    resp = meth(args)
                    if resp is not None:
                        return resp
                elif args.strip():
                    # call method with args
                    resp = meth(json.loads(args))
                    if resp is not None:
                        return resp
                else:
                    resp = meth(args)
                    if resp is not None:
                        return resp
            else:
                return f"Unknown Function: {function_name}"
        



from mechanician.ai_connectors import AIConnector
from mechanician.ai_tools import AITools
import json
import os
import logging

logger = logging.getLogger(__name__)

class TAGAI():

    ###############################################################################
    ## INIT
    ###############################################################################

    def __init__(self,
                 ai_connector: 'AIConnector',
                 ai_instructions=None, 
                 tool_instructions=None,
                 instruction_set_directory=None,
                #  instruction_set_file_name="instructions.json",
                 tool_instruction_file_name="tool_instructions.json",
                 ai_instruction_file_name="ai_instructions.md",
                 tools: 'AITools'=None, 
                 name="Mechanician AI"):
        self.ai_connector = ai_connector
        self.name = name
        self.RUNNING = False
        self.tools = tools
        self.ai_instructions = ai_instructions
        self.tool_instructions = tool_instructions
        if (instruction_set_directory is not None) and (ai_instructions is None):
            self.instruction_set_directory = instruction_set_directory
            self.ai_instruction_file_name = ai_instruction_file_name
            self.load_ai_instructions(instruction_set_directory, ai_instruction_file_name)

        if (instruction_set_directory is not None) and (tool_instructions is None):
            self.instruction_set_directory = instruction_set_directory
            self.tool_instruction_file_name = tool_instruction_file_name
            self.load_tool_instructions(instruction_set_directory, tool_instruction_file_name)

        self._instruct(ai_instructions=self.ai_instructions, 
                       tool_instructions=self.tool_instructions,
                       tools = tools)
        self.ai_connector._connect()

    ###############################################################################
    ## LOAD INSTRUCTIONS
    ###############################################################################

    def load_ai_instructions(self, instruction_set_directory, ai_instruction_file_name):
        ai_instruction_path = os.path.join(instruction_set_directory, ai_instruction_file_name)
        if os.path.exists(ai_instruction_path):
            with open(ai_instruction_path, 'r') as file:
                logger.info(f"Loading AI Instructions from {ai_instruction_path}")
                ai_instructions = file.read()
            if self.ai_instructions is None:
                self.ai_instructions = ai_instructions
        else:
            logger.info(f"AI Instruction file not found at {ai_instruction_path}")
            logger.info("AI Instructions will not be loaded from file")

    def load_tool_instructions(self, instruction_set_directory, tool_instruction_file_name):
        tool_instruction_path = os.path.join(instruction_set_directory, tool_instruction_file_name)
        if os.path.exists(tool_instruction_path):
            with open(tool_instruction_path, 'r') as file:
                logger.info(f"Loading Tool Instructions from {tool_instruction_path}")
                tool_instructions = json.loads(file.read())
            if self.tool_instructions is None:
                self.tool_instructions = tool_instructions
        else:
            logger.info(f"Tool Instruction file not found at {tool_instruction_path}")
            logger.info("Tool Instructions will not be loaded from file")


    ###############################################################################
    ## INSTRUCT
    ###############################################################################

    def _instruct(self, ai_instructions=None, 
                 tool_instructions=None,
                 tools: 'AITools'=None):
        if ai_instructions is not None:
            self.ai_instructions = ai_instructions

        if tool_instructions is not None:
            self.tool_instructions = tool_instructions

        if tools is not None:
            self.tools = tools

        self.ai_connector._instruct(ai_instructions=ai_instructions, 
                                    tool_instructions=tool_instructions,
                                    tools = tools)
    
    ###############################################################################
    # TUNING SESSION TRANSCRIBER FUNCTIONS
    ###############################################################################

    def get_tuning_session(self):
        tuning_session = {}
        tuning_session["tool_instructions"] = self.tool_instructions
        tuning_session["ai_instructions"] = self.ai_instructions
        tuning_session["transcript"] = self.get_message_history()
        tuning_session["test_results"] = None
        return json.dumps(tuning_session, indent=2)


    def save_tuning_session(self, 
                            tuning_session_dir="./tuning_sessions", 
                            file_name="tuning_session.json"):
        if tuning_session_dir is not None:
                os.makedirs(tuning_session_dir, exist_ok=True)
        with open(os.path.join(tuning_session_dir, file_name), 'w') as f:
            json.dump(self.get_tuning_session(), f, indent=2)



    ###############################################################################
    ## SUBMIT_PROMPT
    ###############################################################################

    def submit_prompt(self, prompt):
        return self.ai_connector.submit_prompt(prompt)
    

    ###############################################################################
    ## GET_MESSAGE_HISTORY
    ###############################################################################
    def get_message_history(self):
        return self.ai_connector.get_message_history()
    

    ###############################################################################
    ## STREAMING
    ###############################################################################
    def streaming_connector(self):
        return self.ai_connector.STREAMING


    ###############################################################################
    ## CLEAN UP
    ###############################################################################

    def clean_up(self):
        return self.ai_connector.clean_up()


from mechanician import TAGAI
from mechanician.util import print_markdown
from rich.console import Console 
import logging
import subprocess

logger = logging.getLogger(__name__)
console = Console()

###############################################################################
## PREPRCOESS_PROMPT
###############################################################################

def preprocess_prompt(ai: 'TAGAI', prompt: str):

    if prompt.startswith('/file'):
        filename = prompt.replace('/file ', '', 1)

        with open(filename, 'r') as file:
            prompt = file.read()
            print('')
            print_markdown(console, "------------------")
            print_markdown(console, "## INPUT FILE")
            print_markdown(console, f"``` \n{prompt}\n ```")
            print_markdown(console, "------------------")
            print('')

    elif prompt.startswith('/bye'):
        ai.RUNNING = False
        prompt = ''

    elif prompt.startswith('/$'):
        subprocess.run(prompt.replace('/$', '', 1), shell=True)
        # Return an empty prompt so that it's skipped
        return ''

    return prompt


###############################################################################
## PRINT_HEADER
###############################################################################

def print_header(name):
    print('\n\n\n')
    print_markdown(console, f"# {name}")
    print('\n\n\n')


###############################################################################
## RUN
###############################################################################

def run(ai: TAGAI):
    print_header(name=ai.name)
    # Loop while RUNNING is True, processing user input from the terminal
    ai.RUNNING = True
    try:
        while ai.RUNNING is True:
            # Get the user's prompt
            prompt = input("> ")

            # Skip empty prompts
            if prompt == '':
                continue

            print('')
            prompt = preprocess_prompt(ai, prompt)
            # If preprocessed prompt is None, we should skip it
            if prompt == '':
                continue

            resp = ai.submit_prompt(prompt)

            if not ai.streaming_connector():
                print_markdown(console, resp)
                print('')

            # resp = None, tool_calls were processed and we need to get a new stream to see the model's response
            # This should never happen with the Assistant API, just in the Chat API
            while resp == None:
                resp = ai.submit_prompt(None)

            print('\n')

        logging.info(f"Exiting {ai.name}...")
        return ai
            
    except KeyboardInterrupt:
        logging.info("Ctrl+C was pressed, exiting...")
    except EOFError:
        logging.info("Ctrl+D was pressed, exiting...")
    finally:
        ai.clean_up()
        logging.info("goodbye")




from abc import ABC, abstractmethod
from rich.markdown import Markdown
from rich.console import Console 
import logging

logger = logging.getLogger(__name__)

###############################################################################
## STREAM PRINTER
###############################################################################
        

class StreamPrinter(ABC):

    @abstractmethod
    def print(self, text):
        pass



class SimpleStreamPrinter(StreamPrinter):
    
    def print(self, text, end='\n', flush=True):
        print(text, end=end, flush=flush)
        return
  

def print_markdown(console, text):
    console.print(Markdown(text))
    return


import json
from mechanician import TAGAI
from mechanician.ai_connectors import AIConnector
from mechanician.ai_tools import AITools
from mechanician.util import print_markdown
from rich.console import Console
import logging
import os
import datetime

logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)
handler = logging.StreamHandler()
handler.setLevel(logging.DEBUG)
logger.addHandler(handler)

tuner_instructions = """
Your role is an Instructor AI that will evaluate the performance of an AI Assistant given its instructions,
the tool instructions it was provided, and a transcript of its interactions with a user.

You will be provided a JSON document containing tuning session data, it includes:

ai_instructions: Instructions provided to the AI describing it role
transcript: a transcript of the interaction between the ASSISTANT, the EVALUATOR, and the TOOLS the ASSISTANT used
tool_instructions: A description of each tool and its parameters
test_results: If any where available

Please identify any errors made by the Assistant, as revealed through feedback within the transcript or from tool outcomes. 
Also, highlight behaviors or responses that were useful or explicitly requested by the user.

Once you have completed your evaluation, use the 'draft_instructions' tool you have been provided to a CONSISE UPDATE to the existing 'ai_instructions'
that will prevent the errors you have identified while maintaining the intended role of the ASSISTANT.

BE SPECIFIC to the AI's role and the tools it uses when revising its instructions.


"""

###############################################################################
## INSTRUCTION AUTO TUNING AI
###############################################################################

def instruction_auto_tuning_ai(ai_connector: AIConnector,
                               tuning_session_dir="./tuning_sessions",
                               instructions_dir="./instructions"):
    
    tools = AutoTuningAITools(tuning_session_dir=tuning_session_dir,
                              instructions_dir=instructions_dir)
    
    ai = TAGAI(ai_connector=ai_connector, 
               ai_instructions=tuner_instructions, 
               tool_instructions=auto_tuner_tool_instructions,
               tools=tools,
               name="Instruction Auto-Tuner AI")
    return ai


###############################################################################
# AutoTuningAITools
###############################################################################

class AutoTuningAITools(AITools):

    def __init__(self, 
                 tuning_session_dir=None,
                 instructions_dir=None,
                 ai_instructions_file_name="ai_instructions.md",
                 tool_instructions_file_name="tool_instructions.json"):
        
        self.revised_ai_instructions = None
        self.tuning_session_dir = tuning_session_dir
        self.instructions_dir = instructions_dir

        self.ai_instructions_file_name = ai_instructions_file_name
        self.tool_instructions_file_name = tool_instructions_file_name

        self.tuning_session_file_name = "tuning_session.json"

        self.draft_ai_instructions_file_name = f"draft_{self.ai_instructions_file_name}"
        self.draft_tool_instructions_file_name = f"draft_{self.tool_instructions_file_name}"

        self.ai_instructions_file_path = os.path.join(self.instructions_dir, self.ai_instructions_file_name)
        self.tool_instructions_file_path = os.path.join(self.instructions_dir, self.tool_instructions_file_name)

        self.draft_ai_instructions_file_path = os.path.join(self.instructions_dir, self.draft_ai_instructions_file_name)
        self.draft_tool_instructions_file_path = os.path.join(self.instructions_dir, self.draft_tool_instructions_file_name)

        self.tuning_session_file_path = os.path.join(self.tuning_session_dir, self.tuning_session_file_name)
        self.tuning_session = None

        # ensure the directories exist and set them up if they don't
        if self.tuning_session_dir is not None:
            os.makedirs(self.tuning_session_dir, exist_ok=True)
        if self.instructions_dir is not None:
            os.makedirs(self.instructions_dir, exist_ok=True)

        if os.path.exists(self.ai_instructions_file_path):
            with open(self.ai_instructions_file_path, 'r') as f:
                self.ai_instructions = f.read()
        else:
            self.ai_instructions = None

        if os.path.exists(self.tool_instructions_file_path):
            with open(self.tool_instructions_file_path, 'r') as f:
                self.tool_instructions = json.load(f)
        else:
            self.tool_instructions = []

        # load the tuning session if it exists
        if os.path.exists(self.tuning_session_file_path):
            with open(self.tuning_session_file_path, 'r') as f:
                self.tuning_session = json.load(f)

        # If the tool_instructions is [], set them to the 'tool_instruction' field of the tuning_session object.
        if not self.tool_instructions and self.tuning_session is not None:
            self.tool_instructions = self.tuning_session.get("tool_instructions")

        if not self.ai_instructions and self.tuning_session is not None:
            self.ai_instructions = self.tuning_session.get("ai_instructions")


    def draft_ai_instructions(self, input):
        try:
            self.revised_ai_instructions = input.get("revised_ai_instructions")
            if self.revised_ai_instructions is None:
                resp = "revised_ai_instructions parameter is required"
                logger.info(resp)
                return resp
            console = Console()
            print_markdown(console, "# REVISED INSTRUCTIONS:")
            print_markdown(console, f"{self.revised_ai_instructions}")
            # Write out the revised instructions to the instructions file
            self.ai_instructions = self.revised_ai_instructions

            with open(os.path.join(self.instructions_dir, self.draft_ai_instructions_file_name), 'w') as f:
                f.write(self.ai_instructions)

            resp = "AI Instructions revised successfully"
            logger.info(resp)
            return resp
        except Exception as e:
            message = str(e)
            logger.error(f"ERROR: {message}")
            return f"ERROR: {message}"
        

    def draft_tool_instructions(self, input):
        try:
            tool_name = input.get("tool_name")
            revised_tool_instructions = input.get("revised_tool_instructions")
            if revised_tool_instructions is None or tool_name is None:
                resp = "revised_tool_instructions and tool_name parameters are required"
                logger.info(resp)
                return resp
            console = Console()
            print_markdown(console, f"# REVISED TOOL INSTRUCTIONS FOR {tool_name}:")
            print_markdown(console, f"{revised_tool_instructions}")
            # write out the revised tool instructions to the correct tool in the tool_instructions list
            for tool in self.tool_instructions:
                if tool.get("function").get("name") == tool_name:
                    tool["function"]["description"] = revised_tool_instructions
                    break
            
            with open(os.path.join(self.instructions_dir, self.draft_tool_instructions_file_name), 'w') as f:
                logger.debug("Revised Tool Instructions:")
                logger.debug(self.tool_instructions)
                json.dump(self.tool_instructions, f, indent=2)

            resp = "Tool instructions revised successfully"
            logger.info(resp)
            return resp
        except Exception as e:
            message = str(e)
            logger.error(f"ERROR: {message}")
            return f"ERROR: {message}"
        

    def draft_tool_parameter_instructions(self, input):
        try:
            tool_name = input.get("tool_name")
            parameter_name = input.get("parameter_name")
            revised_parameter_instructions = input.get("revised_parameter_instructions")
            if revised_parameter_instructions is None or tool_name is None or parameter_name is None:
                resp = "revised_parameter_instructions, tool_name, and parameter_name parameters are required"
                logger.info(resp)
                return resp
            console = Console()
            print_markdown(console, f"# REVISED TOOL PARAMETER INSTRUCTIONS FOR {tool_name}/{parameter_name}:")
            print_markdown(console, f"{revised_parameter_instructions}")
            # write out the revised tool parameter instructions to the correct tool in the tool_instructions list
            for tool in self.tool_instructions:
                if tool.get("function").get("name") == tool_name:
                    for parameter in tool.get("function").get("parameters").get("properties"):
                        if parameter == parameter_name:
                            tool["function"]["parameters"]["properties"][parameter]["description"] = revised_parameter_instructions
                            break
                    break
            with open(os.path.join(self.instructions_dir, self.draft_tool_instructions_file_name), 'w') as f:
                logger.debug("Revised Tool Parameter Instructions:")
                logger.debug(self.tool_instructions)
                json.dump(self.tool_instructions, f, indent=2)

            resp = "Tool instructions revised successfully"
            logger.info(resp)
            return resp
        except Exception as e:
            message = str(e)
            logger.error(f"ERROR: {message}")
            return f"ERROR: {message}"


    def commit_draft_instructions(self, input):
        try:
            resp = ""
            # Get the current timestamp
            timestamp = datetime.datetime.now().strftime("%Y%m%d%H%M%S")
            # Create archives directory if it doesn't exist
            archive_dir = os.path.join(self.instructions_dir, f"archives/instructions_{timestamp}")
            os.makedirs(archive_dir, exist_ok=True)
            
            if os.path.exists(self.draft_ai_instructions_file_path):
              # If previous version of instruction file exists
              if os.path.exists(self.ai_instructions_file_path):
                # Specify the new directory and new name
                new_name = os.path.join(archive_dir, self.ai_instructions_file_name)
                # Rename the old file, moving into the archives directory
                os.rename(self.ai_instructions_file_path, new_name)
              
              # Rename the new file
              os.rename(self.draft_ai_instructions_file_path, self.ai_instructions_file_path)
              resp = "Draft AI Instructions saved successfully.\n"

            if os.path.exists(self.draft_tool_instructions_file_path):
              # If previous version of tool instruction file exists
              if os.path.exists(self.tool_instructions_file_path):
                # Specify the new directory and new name
                new_name = os.path.join(archive_dir, self.tool_instructions_file_name)
                # Rename the old file, moving into the archives directory
                os.rename(self.tool_instructions_file_path, new_name)
              
              # Rename the new file
              os.rename(self.draft_tool_instructions_file_path, self.tool_instructions_file_path)  
              resp += "Draft Tool Instructions saved successfully\n"            

            if resp == "":
                resp = "Draft instructions files were not found"

            logger.info(resp)
            return resp
        
        except Exception as e:
            message = str(e)
            logger.error(f"ERROR: {message}")
            return f"ERROR: {message}"

    

###############################################################################
# Auto Tuner Tool Instructions
###############################################################################
auto_tuner_tool_instructions = [
    {
      "type": "function",
      "function": {
        "name": "draft_ai_instructions",
        "description": "Revises an AI's system instructions.",
        "parameters": {
          "type": "object",
          "properties": {
            "revised_ai_instructions": {
              "type": "string",
              "description": "The revised instructions for the AI."
            }
          },
          "required": ["revised_instructions"]
        }
      }
    },
    {
      "type": "function",
      "function": {
        "name": "draft_tool_instructions",
        "description": "Revises Tool instructions.",
        "parameters": {
          "type": "object",
          "properties": {
            "revised_tool_instructions": {
              "type": "string",
              "description": "The revised instructions for the tool."
            },
            "tool_name": {
              "type": "string",
              "description": "The name of the tool to have its instructions revised."
            }
          },
          "required": ["tool_name", "revised_tool_instructions"]
        }
      }
    },
    {
      "type": "function",
      "function": {
        "name": "draft_tool_parameter_instructions",
        "description": "Revises the parameter instructions for a tool.",
        "parameters": {
          "type": "object",
          "properties": {
            "revised_parameter_instructions": {
              "type": "string",
              "description": "The revised tool parameter instructions for the assistant."
            },
            "parameter_name": {
              "type": "string",
              "description": "The name of the parameter to have its instructions revised."
            },
            "tool_name": {
              "type": "string",
              "description": "The name of the tool's whose parameter will have its instructions revised."
            }
          },
          "required": ["parameter_name", "tool_name", "revised_parameter_instructions"]
        }
      }
    },
    {
      "type": "function",
      "function": {
        "name": "commit_draft_instructions",
        "description": "Commits the draft of the new instructions, replacing the original instructions after making an archive copy of the original instructions.",
        "parameters": {
          "type": "object",
          "properties": {},
          "required": []
        }
      }
    },
  ]



from mechanician.util import print_markdown
# from mechanician.ai_connectors import AIConnector
from mechanician import TAGAI
from rich.console import Console
from typing import List
import logging

logger = logging.getLogger(__name__)

###############################################################################
## TEST CLASS
###############################################################################

class QandATest:
    prompt = None
    expected = None
    actual = None
    evaluation = None

    def __init__(self, prompt, expected):
        self.prompt = prompt
        self.expected = expected
        self.actual = None
        self.evaluation = None

    def to_dict(self):
        return {"prompt": self.prompt, "expected": self.expected, "actual": self.actual, "evaluation": self.evaluation}
    
    def __repr__(self):
        return f"QandATest(prompt={self.prompt}, expected={self.expected}, actual={self.actual}, evaluation={self.evaluation})"
    

###############################################################################
## RUN_Q_AND_A_EVALUATIONS
###############################################################################

def run_q_and_a_evaluations(ai: TAGAI, tests: List[QandATest], ai_evaluator:TAGAI=None):
    # If ai_evaluator is None, have the ai self-evaluate
    if ai_evaluator is None:
        ai_evaluator = ai

    messages = []
    results = []
    console = Console()

    try:
        for test in tests:
            prompt = test.prompt
            print_markdown(console, "------------------")
            print_markdown(console, "## TEST")
            print_markdown(console, f"```{prompt}```")
            messages.append(f"EVALUATOR: {prompt}")
            resp = ai.submit_prompt(test.prompt)

            if not ai.streaming_connector():
                print_markdown(console, resp)
                print('\n\n\n')

            # resp = None, tool_calls were processed and we need to get a new stream to see the model's response
            while resp == None:
                resp = ai.submit_prompt(None)
                # print('')

            # Record the response as the ACTUAL response
            test.actual = resp
            messages.append(f"ASSISTANT: {resp}")
            eval_instructions = """Below is an EXPECTED response to a question and an ACTUAL response given by a test taker to the same question. Does the ACTUAL response provide the expected answer? Respond with PASS or FAIL only."""

            print_markdown(console, f"* EVAL INSTRUCTIONS: ```{eval_instructions}```")
            print_markdown(console, f"* EXPECTED: ```{test.expected}```")
            print_markdown(console, f"* ACTUAL: ```{test.actual}```")
            print('\n')

            # Submit the evaluation prompt
            eval_prompt = eval_instructions
            eval_prompt += f"* EXPECTED RESPONSE: \"{test.expected}\"\n\n"
            eval_prompt += f"* ACTUAL RESPONSE: \"{test.actual}\"\n"
            # submit evaluation prompt to ai_evaluator
            resp = ai_evaluator.submit_prompt(eval_prompt)
            test.evaluation = resp
            results.append(test)
            print()
            print_markdown(console, "------------------")
            print('\n')

        logging.info(f"Examinee Exiting ({ai.name})...")
        logging.info(f"Evaluator Exiting ({ai_evaluator.name})...")

        return results, messages
            
    except KeyboardInterrupt:
        logging.info("Ctrl+C was pressed, exiting...")
    except EOFError:
        logging.info("Ctrl+D was pressed, exiting...")
    finally:
        ai.clean_up()
        logging.info("goodbye")





###############################################################################
## RUN_TASK_EVALUATION
###############################################################################

def run_task_evaluation(ai: TAGAI, ai_evaluator: TAGAI, start_prompt: str="START"):
    messages = []
    console = Console()
    prompt = start_prompt
    RUNNING = True
    print(f"START PROMPT > {start_prompt}")
    EVALUATION = None
    try:
        while RUNNING is True:
            print("\n\n")
            print("EVALUATOR:")
            eval_resp = ai_evaluator.submit_prompt(prompt)
            messages.append(f"EVALUATOR: {eval_resp}")
            print("\n\n")

            # Exit if the evaluator says "PASS" or "FAIL"
            if eval_resp.startswith("PASS"):
                RUNNING = False
                EVALUATION = "PASS"
            elif eval_resp.startswith("FAIL"):
                RUNNING = False
                EVALUATION = "FAIL"

            print("ASSISTANT:")
            assist_resp = ai.submit_prompt(eval_resp)
            messages.append(f"ASSISTANT: {assist_resp}")

            if (not ai.streaming_connector()) and (assist_resp is not None):
                print_markdown(console, f"**ASSISTANT** {assist_resp}")

            # if resp = None, tool_calls were processed and we need to get a new stream to see the model's response
            while assist_resp == None:
                assist_resp = ai.submit_prompt(None)

            print('\n\n\n')
            messages.append(f"ASSISTANT: {assist_resp}")
        
            prompt = assist_resp
            
        return EVALUATION, messages
    
    except KeyboardInterrupt:
        print("Ctrl+C was pressed, exiting...")
    except EOFError:
        print("Ctrl+D was pressed, exiting...")
    finally:
        ai.clean_up()
        print("goodbye")



from setuptools import setup, find_packages

setup(
    name='mechanician',
    version='0.1',
    packages=find_packages(),
    install_requires=[
        'rich>=10.0.0',
        'pyyaml>=5.4.1',
    ],
    author='David Edgar Liebke',
    author_email='david@liebke.ai',
    description='Building tools that use AI and building tools that AIs use.',
    long_description=open('../README.md').read(),
    long_description_content_type='text/markdown',
    url='https://github.com/liebke/mechanician',
)



from abc import ABC, abstractmethod
from mechanician.util import StreamPrinter
from mechanician.ai_tools import AITools
import logging

logger = logging.getLogger(__name__)

class AIConnector(ABC):

    messages = []
    instructions = None
    tool_schemas = None

    @abstractmethod
    def __init__(self, instructions, tool_schemas, 
                 function_handler : 'AITools', name):
        pass

    @abstractmethod
    def submit_prompt(self, prompt) -> str:
        pass

    @abstractmethod
    def clean_up(self) -> None:
        pass

    def get_messages(self):
        return self.messages



class StreamingAIConnector(AIConnector):

    @abstractmethod
    def __init__(self, instructions, tool_schemas, 
                 function_handler : 'AITools', 
                 stream_printer : 'StreamPrinter',
                 name: str):
        pass

    @abstractmethod
    def get_stream(self, prompt):
        pass

    @abstractmethod
    def process_stream(self, stream):
        pass

    @abstractmethod
    def clean_up(self) -> None:
        pass

    def submit_prompt(self, prompt):
        return self.process_stream(self.get_stream(prompt))